Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Module user_supplied_function
    ' Copyright Numerical Algorithms Group 2012

    ' The user-supplied function qphess.
    ' qphess computes the vector  Hx = (H)*x  for some matrix H
    ' that defines the Hessian of the required QP problem.
    ' In this version qphess the Hessian matrix is implicit.
    ' The array h[] is not accessed. There is no special coding
    ' for the case jthcol > 0.
    '
    ' Note that array arguments cannot be passed directly from
    ' the NAG C Library DLL to Visual Basic in a way in which
    ' they can be accessed. Instead, we need to pass pointers
    ' to the arrays, then use the Marshal.Copy function to
    ' get hold of the actual contents.

    ' Skeleton for call-back function qphess_user of type NAG_E04NFC_FUN
    ' Sub qphess_user( _
    '   ByVal n As Integer, _
    '   ByVal jthcol As Integer, _
    '   ByVal ptr_h As IntPtr, _
    '   ByVal tdh As Integer, _
    '   ByVal ptr_x As IntPtr, _
    '   ByVal ptr_hx As IntPtr, _
    '   ByRef comm As Nag_Comm _
    ' )
    ' Insert code here.
    ' End Sub

    Sub qphess(ByVal n As Integer, ByVal jthcol As Integer, ByVal h_ptr As IntPtr, ByVal tdh As Integer, ByVal x_ptr As IntPtr, ByVal hx_ptr As IntPtr, ByRef comm_ptr As Nag_Comm)
        Dim i, j As Integer
        Dim sum As Double
        Dim x(n - 1) As Double
        Dim h(n - 1, tdh - 1) As Double
        Dim hx(n - 1) As Double

        ' Copy from the input array pointer x_ptr to our local array X.
        Call Marshal.Copy(x_ptr, x, 0, n)

        ' In this version of qphess we do not access elements of h
        ' explicitly. Neither do we need the contents of the
        ' Nag_Comm structure

        ' Calculate the vector H*x, in array hx.
        sum = 0.0#
        For i = 0 To n - 1
            sum = 1.69 * x(i)
            For j = 0 To n - 1
                sum = sum + x(j) * Math.Abs(i - j)
            Next j
            hx(i) = sum
        Next i

        ' Copy the array hx back to the NAG C Library calling function.
        Call Marshal.Copy(hx, 0, hx_ptr, n)

    End Sub
End Module